pub enum Nullable<T> {
Valid(T),
Null,
}Variants
Implementations
Sourceimpl<T> Nullable<T>
impl<T> Nullable<T>
pub fn into_option(self) -> Option<T>
pub fn as_option(&self) -> Option<&T>
pub fn as_option_mut(&mut self) -> Option<&mut T>
pub fn is_valid(&self) -> bool
pub fn is_null(&self) -> bool
Sourcepub fn expect<'a, A>(self, msg: &'a A) -> T
pub fn expect<'a, A>(self, msg: &'a A) -> T
Return value if this enum is Valid(value), otherwise panics with
the error message msg.
Sourcepub fn valid_or<E>(self, err: E) -> Result<T, E>
pub fn valid_or<E>(self, err: E) -> Result<T, E>
Convert to a result, returning err if this enum is Null.
Sourcepub fn map<U, F>(self, func: F) -> Nullable<U>where
F: FnOnce(T) -> U,
pub fn map<U, F>(self, func: F) -> Nullable<U>where
F: FnOnce(T) -> U,
Maps the T value inside a Nullable<T>::Valid into the U of
a Nullable<U>::Valid using the provided func argument,
or returns Null if this enum is Null.
Sourcepub fn and_then<U, F>(self, func: F) -> Nullable<U>
pub fn and_then<U, F>(self, func: F) -> Nullable<U>
Returns Null if this value is Null, or, if this enum is
Valid(value), it calls func(value) and returns func’s result.
This is very similar to Nullable::map(). However, map()
maps the value inside a Valid(value) to another Valid(value),
while and_then() can be used to reject a valid value and return
null anyway.