Struct pgrx::pg_sys::NullableDatum
source · #[repr(C)]pub struct NullableDatum {
pub value: Datum,
pub isnull: bool,
}Expand description
This struct consists of a Datum and a bool, matching Postgres’s definition as of Postgres 12. This isn’t efficient in terms of storage size, due to padding, but sometimes it’s more cache-friendly, so sometimes it is the preferred type.
Fields
value: Datumisnull: boolTrait Implementations
sourceimpl Clone for NullableDatum
impl Clone for NullableDatum
sourcefn clone(&self) -> NullableDatum
fn clone(&self) -> NullableDatum
Returns a copy of the value. Read more
1.0.0 · sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moresourceimpl Debug for NullableDatum
impl Debug for NullableDatum
sourceimpl TryFrom<NullableDatum> for Datum
impl TryFrom<NullableDatum> for Datum
impl Copy for NullableDatum